'''
# ----------------------------------------------------------------------

 Filter to create a new image applying a HEDCUT LOOK to the INPUT LAYER
 SE=versiopn without the use of G'MIC filters
 
 ...................................................................... 
 
 Release 1.0   initial
 
 Flow implemented to get the final result:      
 
 1. save context values (restored at the end)
 2. create the output image (size is a user option),
    BG as a copy of the input layer
 3. apply Curve Values modification, if chosen
 4. create an Edges Layer, applying Gimp Difference of Gaussian
 5. create a Texture Layer, using :
    - RobA Mosaic Tile Helper, modifying the result using plug-ins waves, erode and erase every-other-row
    - adding a pattern for a Dirty aspect  
    - adding a Canvas aspect
 6. create a second Texture Layer:   
    - using Gimpressionist with a new preset for Pen Drawing
 7. blur the first texture layer
 8. create n Convolution Layers (where n=intensity chosen)
    the convolution layer is created by:
    - copy the BG (normal)
    - copy again and apply blur, desaturate and stretch HSV
    - set mode to Grain Extract and merge down
    - set merged layer mode to Grain Merge
 9. create a New from Visible layer 
10. create 2 Etching Layer, using Gimpressionist with two new presets
    only the lines are kept (using color-to-alpha)
11. create a Colour layer, copying the Source if this option has been chosen
12. create a New from Visible layer, which may or may not include effect of Colour Layer
13. create a Final Outcome, which can be used to apply the final Colouring Option chosen (Source, Grayscale, InvertedColours, Golden)
14. duplicate and merge, if option chosen, applying a Mode correlated to the Mean value of the Outcome
15. create 4 new outcomes representing all the 4 options, if chosen
16. flatten the work image if chosen
    when both "create 4 outcomes" and "flatten=True" are chosen the work image is removed, otherwise it would be a useless duplicate)
 
 ------------------------------------------------------------------
 COPYRIGHT NOTICE
 ------------------------------------------------------------------
 
 This program is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation either version 3 of the License, or
 (at your option) any later version.  
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program if not, you can view the GNU General Public
 License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html
 Alternatively you can write to the Free Software Foundation, Inc., 675 Mass
 Ave, Cambridge, MA 02139, USA.
 
 --------------------------------------------------------------------------------------------------------------------------------------------------
 other info
 Prerequisite: script-fu RobA Mosaic Tle Helper
               pattern "Paper_Iridescent Brush taster.jpg"
               3 new Gimpressionisr Presets: PenDrawing, PenDrawingSlim45, PenDrawingSlim135
 --------------------------------------------------------------------------------------------------------------------------------------------------
 *Lyle'S Tutorial (initial inspiration)

  First, start with your target image and run lylejk stencil with settings below (5, 10, 0.7)
  Lower radius to get the fine detail.
  Play with this setting gives you either wide lines or closer lines in the end.
  
  Then, you run Anisotrophic smoothing preset. I actually have this setting saved as fake smudge but I went to the actually preset for this flow:
  (330,0.36,1.00,0.50,7.38,0.80,30,2,1.....)
  
  Finally, you run the BW Stamp preset; added just a little smoothing: (50, 0.8, 0.8, 0, invert)
  
  By the way, this simple flow works fine for the oranges image, but for a better added step, you can use convolution to amplify the image's essence.
  I use Bilateral filter for this technique.
  After you do the anisotrophic step,...
  
  Copy the original target above your already processed result and duplicate that.
  
  Then run Bilateral preset on the top layer: (10, 12, 2)
  
  You then set the layer property to Grain Extract and merge down. This is your convolution layer.
  You set this resulting layer to Grain Merge and duplicate several times to taste (i did so 5 times).
  Then you flatten and run bw Stamp. Below's that result:....

 --------------------------------------------------------------------------------------------------------------------------------------------------
 the following line terminates the long initial comments
'''

#!/usr/bin/env python
from gimpfu import *
import decimal

############################################################################
# include, then use: erro_msg or info_msg
#---------------------------------------------------------------------------
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    gimp.message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    gimp.message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
        
# ========================================================
# ANCILLARY MODULE : CHECK/PERFORM DUP/MERGE
# --------------------------------------------------------
def check_dup_merge(img,layer,doit,suffix) :
    # ---------------------------------------------------------------------------------------------------
    if doit == 1 :     # increase neatness of final outcome
        DupFinal = layer.copy()
        DupFinal.name = "Dup Final Layer"
        DupFinal.opacity = 100.0 
        img.add_layer(DupFinal, 0)
        
        (mean, stddev, median, pixels, count, percent)=pdb.gimp_histogram(DupFinal, 0, 0, 255)
        if mean > 160 :
            DupFinal.mode =  MULTIPLY_MODE
            suffix = suffix+" (Dupl&Merge, Multiply)"
        elif mean > 90 :
            DupFinal.mode =  SOFTLIGHT_MODE
            suffix = suffix+" (Dupl&Merge, SoftLight)"
        else :
            DupFinal.mode =  SCREEN_MODE
            suffix = suffix+" (Dupl&Merge, Screen)"
        layer = pdb.gimp_image_merge_down (img, DupFinal, 0)
        
    return(layer,suffix)    
        
# ========================================================
# ANCILLARY MODULE : SET EACH PIXEL EITHER BLACK OR TRANSP
# --------------------------------------------------------
def black_or_transp(layer) :
    
    # Analysis of each pixel: set RGB=000 where alpha = 0 
    
    img = layer.image
    # Indicates that the process has started.
    gimp.progress_init("Transforming " + layer.name + " setting pixels black or transparent...")

    # Get the layer position.
    pos = 0;
    for i in range(len(img.layers)):
        if(img.layers[i] == layer):
            pos = i
    
    # Create a new layer to save the results (otherwise is not possible to undo the operation).
     
    newLayer = gimp.Layer(img,  layer.name+"_work", layer.width, layer.height, layer.type, layer.opacity, layer.mode)
    img.add_layer(newLayer, pos)
    layerName = layer.name
    
    # Clear the new layer.
    pdb.gimp_edit_clear(newLayer)
    newLayer.flush()
    
    # Convert the pixels to BLACK or TRANSPARENT.
    try:
        # Calculate the number of tiles.
        tn = int(layer.width / 64)
        if(layer.width % 64 > 0):
            tn += 1
        tm = int(layer.height / 64)
        if(layer.height % 64 > 0):
            tm += 1
        
        # Iterate over the tiles.
        for i in range(tn):
            for j in range(tm):
                # Update the progress bar.
                gimp.progress_update(float(i*tm + j) / float(tn*tm))
        
                # Get the tiles.
                srcTile = layer.get_tile(False, j, i)
                dstTile = newLayer.get_tile(False, j, i)
        
                # Iterate over the pixels of each tile.
                for x in range(srcTile.ewidth):
                    for y in range(srcTile.eheight):
                        # Get the pixel and set alpha to 255.
                        pixel = srcTile[x,y]
                        red = ord(pixel[0])
                        green = ord(pixel[1])
                        blue = ord(pixel[2])
                        transp = ord(pixel[3])
                        #  
                        if transp > 0:
                            newred = 0
                            newgreen = 0
                            mewblue = 0
                            newtransp = 255
                        else:
                            newred = 0
                            newgreen = 0
                            mewblue = 0
                            newtransp = 0
       
                        res = chr(newred) + chr(newgreen) + chr(mewblue) + chr(newtransp)
                        
                        # If the image has other channels channel) copy his values.
                        if(len(pixel) > 4):
                            for k in range(len(pixel)-4):
                                res += pixel[k+4]
                                
                        # Save the value in the result layer.
                        dstTile[x,y] = res
        
        # Update the new layer.
        newLayer.flush()
        newLayer.merge_shadow(True)
        newLayer.update(0, 0, newLayer.width, newLayer.height)
        
        # Remove the old layer.
        img.remove_layer(layer)
        # Change the name of the new layer (two layers can not have the same name).
        newLayer.name = layerName
     
        
    except Exception as err:
        gimp.message("Unexpected error: " + str(err))
    
    # End progress.
    pdb.gimp_progress_end()

    return (newLayer)

############################################################################

def CrazyHedcutSE(inImage, inLayer,
                      widthheight,
                      prep_curves,
                      intensity,
                      source_on_top,
                      which_colours,
                      doDupMerge,
                      doFlatten) :
 
    pdb.gimp_context_push
    currFG = pdb.gimp_context_get_foreground()
    currBG = pdb.gimp_context_get_background()
    
    owidth = inImage.width
    oheight = inImage.height
    type = inImage.base_type
    name = inLayer.name
    if '.' in name:
        split = name.split(".", 1)
        name = split[0]
    suffix = ", as a Crazy Hedcut Drawing (intensity="+str(intensity)+")"
    
    # calculate scale factor for the new output image
    realWidthHeight = owidth+oheight      # real size of the input image
    if widthheight == 0 :   # means the user wants to have same size as input
        widthheight = realWidthHeight                                    
        scale = 1
    else :     
        optiWidthHeight = widthheight       # size to work on to get best results
        scale = decimal.Decimal(optiWidthHeight)/decimal.Decimal(realWidthHeight)      # ratio for scaling

    rwidth=int(owidth*scale)
    rheight=int(oheight*scale)
    # note : original will be scaled back at the end  
    pdb.gimp_image_scale (inImage, rwidth, rheight)
# --------------------------------------------------------------------------------------

    newImage = pdb.gimp_image_new (rwidth, rheight, type)                    
    newBG = pdb.gimp_layer_new_from_drawable(inLayer, newImage)     # background layer
    newBG.mode = NORMAL_MODE
    newBG.name = "Background Image Layer"
    newBG.opacity = 100.0 
    newImage.add_layer (newBG, 0)
    pdb.gimp_layer_scale(newBG, rwidth, rheight, False)
    alpha=pdb.gimp_drawable_has_alpha(newBG)
    if (alpha==False):
        pdb.gimp_layer_add_alpha (newBG)    #  RGBA
    
# =======================================================================================
    
    if prep_curves == 1 :   # moderate brighten
        pdb.gimp_curves_spline (newBG, HISTOGRAM_VALUE, 6,
                                ( 0, 0, 127, 159, 255, 255) 
                               )
        suffix = suffix+", moderately brightened"
    if prep_curves == 2 :   # brighten
        pdb.gimp_curves_spline (newBG, HISTOGRAM_VALUE, 6,
                                ( 0, 0, 127, 191, 255, 255) 
                               )
        suffix = suffix+", brightened"
    if prep_curves == 3 :   # moderate darken
        pdb.gimp_curves_spline (newBG, HISTOGRAM_VALUE, 6,
                                ( 0, 0, 127, 95, 255, 255) 
                               )
        suffix = suffix+", moderately darkened"
    if prep_curves == 4 :   # darken
        pdb.gimp_curves_spline (newBG, HISTOGRAM_VALUE, 6,
                                ( 0, 0, 127, 63, 255, 255) 
                               )
        suffix = suffix+", darkened"
        
    # add evidence of edges
    # ---------------------
    Edges1= newBG.copy()
    Edges1.name =  "Edges Layer"
    newImage.add_layer (Edges1, 0)
    Edges1.mode =  NORMAL_MODE
    Edges2 = newBG.copy()
    Edges2.name =  "Edges Layer 2"
    newImage.add_layer (Edges2, 0)
    Edges2.mode =  DARKEN_ONLY_MODE
    alpha=pdb.gimp_drawable_has_alpha(Edges2)
    if (alpha==True):
        pdb.gimp_layer_flatten (Edges2)    #  RGB (required by subsequent filter)
    pdb.plug_in_dog (newImage, Edges2, 6.0, 1.0, True, True)
    pdb.gimp_threshold (Edges2, 247, 255)
    # ======================================================================================================
    
    EdgesL = pdb.gimp_image_merge_down (newImage, Edges2, EXPAND_AS_NECESSARY)
    
    #pdb.gimp_display_new (newImage)
    #return()
    
    # add "textures"
    # -------------- 
    Texture1 = EdgesL.copy()
    Texture1.name =  "Texture Layer"
    newImage.add_layer (Texture1, 0)
    Texture1.mode =  NORMAL_MODE
    Texture1.mode =  NORMAL_MODE
    Texture2 = newBG.copy()
    Texture2.name =  "Texture Layer 2"
    newImage.add_layer (Texture2, 0)
    Texture2.mode =  DARKEN_ONLY_MODE
    (mean, stddev, median, pixels, count, percent)=pdb.gimp_histogram(Texture1, 0, 0, 255)
    limitBW = mean+25 
    pdb.gimp_threshold (Texture2, limitBW, 255)
    pdb.gimp_image_select_color (newImage, CHANNEL_OP_REPLACE, Texture2, ((255,255,255,255)))
    spacing = widthheight/100
    pdb.gimp_context_set_brush ("2. Hardness 100")
    pdb.script_fu_mosaic_tile_helper (newImage, Texture2, spacing, 0, 0, 1, ((255,0,0,255)))
    pdb.plug_in_colortoalpha (newImage, Texture2, ((255,255,255,255)))
    pdb.gimp_selection_invert (newImage)
    pdb.script_fu_mosaic_tile_helper (newImage, Texture2, spacing, 0, 0, 1, ((255,0,0,255)))
    pdb.plug_in_colortoalpha (newImage, Texture2, ((0,0,0,255)))
    pdb.gimp_selection_none (newImage)
    Texture2 = black_or_transp(Texture2)
    # ======================================================================================================
    pdb.plug_in_waves (newImage, Texture2, 6.0, 3.0, 10.0, 0, 0)
    pdb.plug_in_erode (newImage, Texture2, 1, HISTOGRAM_VALUE, 1.0, 7, 0, 128)
    pdb.script_fu_erase_rows (newImage, Texture2, 1, 0, 0)
    Texture2.mode =  BURN_MODE
    TextureL = pdb.gimp_image_merge_down (newImage, Texture2, EXPAND_AS_NECESSARY)
    pdb.plug_in_erode (newImage, TextureL, 1, HISTOGRAM_VALUE, 1.0, 7, 0, 128)
    pdb.script_fu_erase_rows (newImage, TextureL, 0, 0, 0)
    Dirty = TextureL.copy()
    Dirty.name =  "Texture Layer 2"
    newImage.add_layer (Dirty, 0)
    Dirty.mode =  HARDLIGHT_MODE
    pdb.gimp_context_set_pattern ("Paper_Iridescent Brush taster.jpg")
    pdb.gimp_edit_fill (Dirty, PATTERN_FILL)
    TextureL = pdb.gimp_image_merge_down (newImage, Dirty, EXPAND_AS_NECESSARY)
    # ======================================================================================================
    
    pdb.plug_in_apply_canvas (newImage, TextureL, 0, 3)
    # ======================================================================================================
    
    PendrawL = newBG.copy()
    PendrawL.name =  "Pendraw Layer "
    newImage.add_layer (PendrawL, 0)
    PendrawL.mode =  MULTIPLY_MODE
    alpha=pdb.gimp_drawable_has_alpha(PendrawL)
    if (alpha==False):
        pdb.gimp_layer_add_alpha (PendrawL)    #  RGBA
    Preset = "PenDrawing"
    pdb.plug_in_gimpressionist (newImage, PendrawL, Preset)
    pdb.plug_in_colortoalpha (newImage, PendrawL, ((255,255,255,255)))
    
    # ======================================================================================================
    
    pdb.plug_in_gauss_rle2 (newImage, TextureL, rwidth/300, rheight/300)
    
    #pdb.gimp_display_new (newImage)
    #return()

    # create convolution layers
    # -------------------------
    newBG1 = newBG.copy()
    newBG1.name =  "BG 1.duplicated"
    newImage.add_layer (newBG1, 0)
    pdb.gimp_desaturate_full (newBG1, 2)
    
    newBG2 = newBG.copy()
    newBG2.name =  "BG 2.duplicated"
    newImage.add_layer (newBG2, 0)
    pdb.plug_in_gauss_rle2 (newImage, newBG2, rwidth/150, rheight/150)
    newBG2.mode =  GRAIN_EXTRACT_MODE
    pdb.gimp_desaturate_full (newBG2, 1)
    pdb.plug_in_autostretch_hsv (newImage, newBG2)
    
    convoL1 = pdb.gimp_image_merge_down (newImage, newBG2, EXPAND_AS_NECESSARY)
    convoL1.name =  "1.convolution layer"
    
    convoL1.mode =  GRAIN_MERGE_MODE        # ...with previously created layers (was GRAIN_MERGE)
    
    # as many convolution layers as intensity level chosen
    
    if (intensity>1) :
        convoL2 = convoL1.copy()
        convoL2.name =  "2.convolution layer"
        newImage.add_layer (convoL2, 0)
    if (intensity>2) :
        convoL3 = convoL1.copy()
        convoL3.name =  "3.convolution layer"
        newImage.add_layer (convoL3, 0)

    VisibleL = pdb.gimp_layer_new_from_visible (newImage, newImage, "New from Visible") 
    newImage.add_layer (VisibleL, 0)
    
    #pdb.gimp_display_new (newImage)
    #return()

    # apply etching
    # -------------
    EtchL1 = newBG.copy()
    EtchL1.name =  "Etch Layer"
    newImage.add_layer (EtchL1, 0)
    EtchL1.mode =  NORMAL_MODE
    EtchL1.opacity =  75.00
    pdb.gimp_threshold (EtchL1, 111, 255)
    
    Preset = "PenDrawingSlim45"
    pdb.plug_in_gimpressionist (newImage, EtchL1, Preset)
    # ======================================================================================================= 
    
    pdb.plug_in_colortoalpha (newImage, EtchL1, (255,255,255,255))
    
    EtchL2 = newBG.copy()
    EtchL2.name =  "Etch Layer 2"
    newImage.add_layer (EtchL2, 0)
    EtchL2.mode =  NORMAL_MODE
    EtchL2.opacity =  75.00
    pdb.gimp_threshold (EtchL2, 143, 255)
    pdb.plug_in_vinvert (newImage, EtchL2)
    
    Preset = "PenDrawingSlim135"
    pdb.plug_in_gimpressionist (newImage, EtchL2, Preset)
    # ======================================================================================================= 
    pdb.plug_in_colortoalpha (newImage, EtchL2, (255,255,255,255))
    
    #pdb.gimp_display_new (newImage)
    #return
    
    if (source_on_top == True) :
        # add colour  
        ColourL = newBG.copy()
        ColourL.name =  "Colour Layer"
        ColourL.mode = GRAIN_MERGE_MODE
        ColourL.opacity = 44.4
        newImage.add_layer (ColourL, 0)
        suffix = suffix+" (Source on top)"

    TopL = pdb.gimp_layer_new_from_visible (newImage, newImage, "Top from Visible") 
    newImage.add_layer (TopL, 0)
    TopL.opacity = 50.0
    TopL.mode = DODGE_MODE  # was MULTIPLY
    
    # additions for final outcome
    FinalL = pdb.gimp_layer_new_from_visible (newImage, newImage, "Final Outcome") 
    newImage.add_layer (FinalL, 0)
    FinalL.opacity = 100.0
    FinalL.mode = NORMAL_MODE
    
    # optional color effect on the final outcome
    # ------------------------------------------
    if which_colours == 0 :     # source
        (FinalL, suffix) = check_dup_merge (newImage, FinalL, doDupMerge, suffix)
    if which_colours == 1 :     # grayscale
        pdb.gimp_desaturate_full (FinalL, 2)
        suffix = suffix+" (grayscale)"
        (FinalL, suffix) = check_dup_merge (newImage, FinalL, doDupMerge, suffix)
    if which_colours == 2 :     # inverted colors
        TIL = FinalL.copy()
        TIL.mode = HUE_MODE       
        TIL.name = "Temp Inverted Layer"
        TIL.opacity = 100.0 
        newImage.add_layer(TIL, 0)
        pdb.gimp_invert(TIL)
        FinalL = pdb.gimp_image_merge_down (newImage, TIL, 0)
        suffix = suffix+" (inverted colours)"
        (FinalL, suffix) = check_dup_merge (newImage, FinalL, doDupMerge, suffix)
    if which_colours == 3 :     # golden look
        pdb.gimp_desaturate_full (FinalL, 2)
        Gold = FinalL.copy()
        Gold.mode = COLOR_MODE       # was GRAIN_MERGE
        Gold.name = "Gold Layer"
        Gold.opacity = 100.0 
        newImage.add_layer(Gold, 0)
        pdb.gimp_context_set_pattern ("MyGold_120.jpg")
        pdb.gimp_edit_fill (Gold, PATTERN_FILL)
        FinalL = pdb.gimp_image_merge_down (newImage, Gold, 0)
        suffix = suffix+" (golden look)"
        (FinalL, suffix) = check_dup_merge (newImage, FinalL, doDupMerge, suffix)
        pdb.gimp_curves_spline (FinalL, HISTOGRAM_VALUE, 6,             # Issabella Taste
                                ( 0, 8, 127, 183, 255, 255) 
                               )
    # ---------------------------------------------------------------------------------------------------
        
    # addition start   <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
    if which_colours == 4 :     # all looks, each as a separate image
        
        startsuffix = suffix

        # SOURCE COLOURS
        (FinalL, suffix) = check_dup_merge (newImage, FinalL, doDupMerge, suffix)
        IMAGEASSOURCE = pdb.gimp_image_duplicate (newImage)
        FlatASSOURCE = pdb.gimp_image_flatten(IMAGEASSOURCE)
        FlatASSOURCE.name = name+suffix
        pdb.gimp_display_new (IMAGEASSOURCE)
        
        # GRAYSCALE
        TGL = FinalL.copy()
        TGL.mode = NORMAL_MODE       
        TGL.name = "Temp Gray Layer"
        TGL.opacity = 100.0 
        newImage.add_layer(TGL, 0)
        pdb.gimp_desaturate_full (TGL, 2)
        suffix = startsuffix+" (grayscale)"
        (TGL, suffix) = check_dup_merge (newImage, TGL, doDupMerge, suffix)
        IMAGEGRAY = pdb.gimp_image_duplicate (newImage)
        FlatGRAY = pdb.gimp_image_flatten(IMAGEGRAY)
        FlatGRAY.name = name+suffix
        pdb.gimp_display_new (IMAGEGRAY)
        pdb.gimp_image_remove_layer (newImage, TGL)
        
        # INVERTED
        TFL = FinalL.copy()
        TFL.mode = NORMAL_MODE       
        TFL.name = "Temp Final Layer"
        TFL.opacity = 100.0
        newImage.add_layer(TFL, 0)
        TIL = TFL.copy()
        TIL.mode = HUE_MODE       
        TIL.name = "Temp Inverted Layer"
        TIL.opacity = 100.0 
        newImage.add_layer(TIL, 0)
        pdb.gimp_invert(TIL)
        TFL = pdb.gimp_image_merge_down (newImage, TIL, 0)
        suffix = startsuffix+" (inverted colours)"
        (TFL, suffix) = check_dup_merge (newImage, TFL, doDupMerge, suffix)
        IMAGEINVERT = pdb.gimp_image_duplicate (newImage)
        FlatINVERT = pdb.gimp_image_flatten(IMAGEINVERT)
        FlatINVERT.name = name+suffix
        pdb.gimp_display_new (IMAGEINVERT)
        pdb.gimp_image_remove_layer (newImage, TFL)
        
        # GOLDEN
        TGL2 = FinalL.copy()
        TGL2.mode = NORMAL_MODE       
        TGL2.name = "Temp Gray Layer"
        TGL2.opacity = 100.0 
        newImage.add_layer(TGL2, 0)
        pdb.gimp_desaturate_full (TGL2, 2)
        Gold = TGL2.copy()
        Gold.mode = COLOR_MODE       # was GRAIN_MERGE
        Gold.name = "Gold Layer"
        Gold.opacity = 100.0 
        newImage.add_layer(Gold, 0)
        pdb.gimp_context_set_pattern ("MyGold_120.jpg")
        pdb.gimp_edit_fill (Gold, PATTERN_FILL)
        TGL2 = pdb.gimp_image_merge_down (newImage, Gold, 0)
        suffix = startsuffix+" (golden look)"
        (TGL2, suffix) = check_dup_merge (newImage, TGL2, doDupMerge, suffix)
        pdb.gimp_curves_spline (TGL2, HISTOGRAM_VALUE, 6,             # Issabella Taste
                                ( 0, 8, 127, 183, 255, 255) 
                               )
        IMAGEGOLDEN = pdb.gimp_image_duplicate (newImage)
        FlatGOLDEN = pdb.gimp_image_flatten(IMAGEGOLDEN)
        FlatGOLDEN.name = name+suffix
        pdb.gimp_display_new (IMAGEGOLDEN)
        pdb.gimp_image_remove_layer (newImage, TGL2)

    # addition end     <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

    
    pdb.gimp_displays_flush

    pdb.gimp_context_pop
    pdb.gimp_context_set_foreground(currFG)
    pdb.gimp_context_set_background(currBG)
    
    pdb.gimp_image_scale (inImage, owidth, oheight)     # source back to original size       
    
    if (doFlatten == True) :
        Flat_Layer = pdb.gimp_image_flatten(newImage)
        Flat_Layer.name = name+suffix
        if which_colours < 4 :  # single outcome
            pdb.gimp_display_new (newImage)
            return (Flat_Layer)
        else :
            pdb.gimp_image_delete (newImage)          

    else :
        pdb.gimp_display_new (newImage)
            
    return ()

# =======================================================================
register(
    "CrazyHedcutSE",    
    "to tranform an image in a crazy hedcut drawing (Special Edition)",   
    "This script transforms the input drawable into a crazy hedcut drawing, with or without colours (Special Edition).",
    "Diego", 
    "Diego Nassetti ", 
    "November 2016",
    "CrazyHedcutSE", 
    "RGB*,GRAY*", 
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "idrawable", "Input drawable", None),
      (PF_ADJUSTMENT, "size", "Width+Height of New Image"+"\n (0=same as input)", 0, (0, 9600, 300)),
      (PF_OPTION, "curve", "Curves preprocess?",  0, ("None",
                                                      "Moderate Brighten",
                                                      "Brighten",
                                                      "Moderate Darken",
                                                      "Darken",
                                                     )),
      (PF_ADJUSTMENT, "intensity",  "Intensity of the Effect", 2, (1, 3, 1)),
      (PF_TOGGLE, "source", "Overlay Source?", True),
      (PF_OPTION, "color", "Final Color option",  1, ("Source",
                                                      "Grayscale",
                                                      "Inverted colours",
                                                      "Golden look",
                                                      "All 4 as separate Images",
                                                     )),
      (PF_TOGGLE, "dupfinal", "Duplicate and Merge Final Layer?", False),
      (PF_TOGGLE, "flatten", "Flatten Image?", True),
    ], 
    [
      (PF_DRAWABLE, "odrawable", "Output drawable", None),  
    ],
    CrazyHedcutSE,
    menu="<Image>/Diego/Artistic",
    )

main()